package com.paxos.messaging.alerts

import io.mockk.confirmVerified
import io.mockk.justRun
import io.mockk.mockk
import io.mockk.verify
import org.junit.jupiter.api.DynamicTest
import org.junit.jupiter.api.DynamicTest.dynamicTest
import org.junit.jupiter.api.TestFactory

class AlertSenderTest {
    private val propsToTest = listOf<Pair<String, Map<String, *>>>(
        "no props" to emptyMap<String, String>(),
        "valid props" to (1..2).associate { "key$it" to "value$it" } + ("keynull" to null),
    )

    @TestFactory
    fun `send alert without valid destination`(): List<DynamicTest> {
        val tests = mutableListOf<DynamicTest>()
        Severity.values().forEach { severity ->
            propsToTest.forEach { (_, props) ->
                tests.add(
                    dynamicTest("$severity,$props") {
                        val sender = AlertSender(config = AlertConfig(severityDestinationMap = emptyMap()))
                        val alert = Alert(
                            severity,
                            message = "test alert",
                            props = props,
                            exception = IllegalArgumentException("test"),
                        )
                        val expectedLogLine =
                            "No alert destination config found for severity ${alert.severity} for alert $alert"
                    },
                )
            }
        }
        return tests
    }

    @TestFactory
    fun `send alert with valid destination`(): List<DynamicTest> {
        val tests = mutableListOf<DynamicTest>()
        Severity.values().forEach { severity ->
            propsToTest.forEach { (_, props) ->
                tests.add(
                    dynamicTest("$severity, $props") {
                        val destination = mockk<AlertDestination>() {
                            justRun { send(any()) }
                        }
                        val sender = AlertSender(
                            config = AlertConfig(
                                severityDestinationMap = mapOf(
                                    severity to destination,
                                ),
                            ),
                        )
                        val alert = Alert(
                            severity,
                            message = "test alert",
                            props = props,
                            exception = IllegalArgumentException("test"),
                        )
                        sender.send(alert)
                        verify(exactly = 1) { destination.send(eq(alert.withContext())) }
                        confirmVerified(destination)
                    },
                )
            }
        }
        return tests
    }
}

fun dynamicTest(name: String, block: () -> Unit): DynamicTest =
    dynamicTest(name) {
        block()
    }